package com.gmei.data.dqmp.utils;

import java.text.ParseException;
import java.util.Date;

import org.quartz.CronExpression;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class CrontabUtils {
	
	private static final Logger logger = LoggerFactory.getLogger(CrontabUtils.class);

	/**
	 * 根据crontab表达式判断当前任务是否应该执行【未来1小时以内】
	 * @param crontabStr
	 * @return
	 */
	public static boolean isToRunByTimeRange(Date currentDate,String crontabStr) {
		boolean rs = false;
		long currentDateMilliseconds = currentDate.getTime();
		CronExpression expression = null;
		try {
			expression = new CronExpression(crontabStr);
			Date newDate = expression.getNextValidTimeAfter(currentDate);
			long newDateMilliseconds = newDate.getTime();
			long diffSeconds = (newDateMilliseconds - currentDateMilliseconds)/1000;
			if(diffSeconds < 60 * 60) {
				rs = true;
			}
		} catch (ParseException e) {
			logger.error("Fail to parse cron express", e);
		} catch (Exception e) {
			logger.error("Fail to update rule nextTime", e);
		}
		return rs;
	}
	
	/**
	 * 根据crontab表达式判断当前任务是否应该执行【是否跟当前年月日时分一致】
	 * @param crontabStr
	 * @return
	 */
	public static boolean isToRunByMinites(Date currentDate,String crontabStr) {
		boolean rs = false;
		try {
			CronExpression expression = new CronExpression(crontabStr);
			rs = expression.isSatisfiedBy(currentDate);
		} catch (ParseException e) {
			logger.error("Fail to parse cron express", e);
		} catch (Exception e) {
			logger.error("Fail to update rule nextTime", e);
		}
		return rs;
	}
	
	public static void main(String[] args) {
		Date currentDate = new Date();
		System.out.println(isToRunByTimeRange(currentDate,"0 0 12 25 * ?"));
		System.out.println(isToRunByMinites(currentDate,"* 57 10 25 * ?"));
		System.out.println(isToRunByMinites(currentDate,"* 50 11 ? * WED"));
	}
}
